#ifndef STARSSegment_h
#define STARSSegment_h

#include "interfaces/STARSTypes.h"

class STARS_Segment_t
{
	public:
		virtual ~STARS_Segment_t(){}
		/* get start and end addresses */
		virtual STARS_ea_t get_startEA() = 0;
		virtual STARS_ea_t get_endEA() = 0;
		virtual std::size_t get_seg_size() = 0;

		/* figure out what kind of segment this is */
		virtual bool IsCodeSegment() const = 0;
		virtual bool IsXternSegment() const { return false; };
		virtual bool IsDataSegment() const = 0;
		virtual bool IsCommonSegment() const = 0;
		virtual bool IsBSSSegment() const = 0;

		/* examine segment permissions */
		virtual bool IsReadableSegment() const = 0;
		virtual bool IsWriteableSegment() const = 0;

		/* returns the segment's name in the given buffer */
		virtual STARS_ssize_t GetSegmentName(char* name, int len) = 0;

		// Fetch read-only data values, if any.
		virtual bool GetReadOnlyMem64BitValue(STARS_ea_t MemAddr, uint64_t &Value) const = 0;
		virtual bool GetReadOnlyMem32BitValue(STARS_ea_t MemAddr, uint32_t &Value) const = 0;
		virtual bool GetReadOnlyMem16BitValue(STARS_ea_t MemAddr, uint16_t &Value) const = 0;
		virtual bool GetReadOnlyMem8BitValue(STARS_ea_t MemAddr, uint8_t &Value) const = 0;

		virtual void Dump() { }
};

#endif
