
#ifndef STARS_IRDB_Interface_h
#define STARS_IRDB_Interface_h

#include <stdlib.h>
#include <fstream>
#include <irdb-core>
#include <libgen.h>
#include <assert.h>
#include <MEDS_AnnotationParser.hpp>


namespace STARS
{

	class Option_t
	{
	public:
		Option_t()
			: ShadowFuncPtrs(false), ShadowCriticalArgs(false), ImproveCFG(false), ReducedAnalysis(false), ReducedAnalysisLevel2(false), DeepLoopAnalyses(false), ConstantPropagation(false)
		{ };

		void SetShadowFuncPtrs(bool FlagValue) { ShadowFuncPtrs = FlagValue; };
		void SetShadowCriticalArgs(bool FlagValue) { ShadowCriticalArgs = FlagValue; };
		void SetMaximizeCFGImprovement(bool FlagValue) { ImproveCFG = FlagValue; };
		void SetReducedAnalysis(bool FlagValue) { ReducedAnalysis = FlagValue; };
		void SetReducedAnalysisLevel2(bool FlagValue) { ReducedAnalysisLevel2 = FlagValue; };
		void SetDeepLoopAnalyses(bool FlagValue) { DeepLoopAnalyses = FlagValue; };
		void SetConstantPropagation(bool FlagValue) { ConstantPropagation = FlagValue; };

		bool ShouldSTARSPerformReducedAnalysis(void) const { return ReducedAnalysis; }; // Is program too big for full analysis?
		bool ShouldSTARSPerformLevel2ReducedAnalysis(void) const { return ReducedAnalysisLevel2; }; // Perform expanded but still incomplete analysis
		bool ShouldSTARSPerformFullAnalysis(void) const { return (!(ReducedAnalysis || ReducedAnalysisLevel2)); };
		bool ShouldSTARSShadowFuncPtrs(void) const { return ShadowFuncPtrs; };
		bool ShouldSTARSShadowCriticalArgs(void) const { return ShadowCriticalArgs; };
		bool ShouldSTARSMaximizeCFGImprovement(void) const { return ImproveCFG; };
		bool ShouldSTARSPerformDeepLoopAnalyses(void) const { return DeepLoopAnalyses; };
		bool ShouldSTARSPerformConstantPropagation(void) const { return ConstantPropagation; };

	private:
		bool ShadowFuncPtrs;
		bool ShadowCriticalArgs; // shadow critical args to security-critical library functions
		bool ImproveCFG; // maximize indirect control flow analyses and annotations
		bool ReducedAnalysis;
		bool ReducedAnalysisLevel2;
		bool DeepLoopAnalyses;
		bool ConstantPropagation;
	};

	class IRDB_Interface_t
	{
		public:
			IRDB_Interface_t(/* IRDB_SDK::pqxxDB_t &p_pqxx_interface */)
// 				: pqxx_interface(p_pqxx_interface)
			{
			};

			STARS::Option_t& GetSTARSOptions(void) { return STARSOptions; };
			const STARS::Option_t& GetSTARSOptions(void) const { return STARSOptions; };

			// invoke stars on one file.
			int do_STARS(IRDB_SDK::FileIR_t* firp);
			MEDS_Annotation::MEDS_AnnotationParser & getAnnotations() { return parser; } 
			const MEDS_Annotation::MEDS_AnnotationParser & getAnnotations() const { return parser; } 

		private:
// 			IRDB_SDK::pqxxDB_t &pqxx_interface;
			MEDS_Annotation::MEDS_AnnotationParser parser;
			STARS::Option_t STARSOptions;
	};

}

#endif
